@@ -18,7 +18,7 @@ import './navbar.js'
 import { AnnotationPopover, importAnnotations, exportAnnotations } from './annotations.js'
 import { SelectionPopover } from './selection-tools.js'
 import { ImageViewer } from './image-viewer.js'
-import { formatAuthors, makeBookInfoWindow } from './book-info.js'
+import { formatLanguageMap, formatAuthors, makeBookInfoWindow } from './book-info.js'
 import { themes, invertTheme, themeCssProvider } from './themes.js'
 import { dataStore } from './data.js'
 
@@ -89,7 +89,7 @@ const ViewPreferencesWindow = GObject.registerClass({
         'theme-flow-box',
         'reduce-animation',
     ],
-}, class extends Adw.PreferencesWindow {
+}, class extends Adw.PreferencesDialog {
     constructor(params) {
         super(params)
         this.font_settings.bindProperties({
@@ -261,6 +261,18 @@ GObject.registerClass({
             },
         }))
 
+        // Mouse Forward/Backward
+        const gestureForward = new Gtk.GestureClick()
+        const gestureBack = new Gtk.GestureClick()
+        gestureForward.set_button(9)
+        gestureBack.set_button(8)
+        this.#webView.add_controller(utils.connect(gestureForward,{
+	    'pressed': () => this.#exec('reader.view.history.forward'),
+        }))
+        this.#webView.add_controller(utils.connect(gestureBack,{
+	    'pressed': () => this.#exec('reader.view.history.back'),
+        }))
+
         const applyStyle = () => this.#applyStyle().catch(e => console.error(e))
         this.viewSettings.connectAll(applyStyle)
         this.fontSettings.connectAll(applyStyle)
@@ -397,6 +409,7 @@ GObject.registerClass({
     mediaOverlayStart() { return this.#exec('reader.view.startMediaOverlay') }
     mediaOverlayPause() { return this.#exec('reader.view.mediaOverlay.pause') }
     mediaOverlayResume() { return this.#exec('reader.view.mediaOverlay.resume') }
+    mediaOverlayStop() { return this.#exec('reader.view.mediaOverlay.stop') }
     mediaOverlayPrev() { return this.#exec('reader.view.mediaOverlay.prev') }
     mediaOverlayNext() { return this.#exec('reader.view.mediaOverlay.next') }
     mediaOverlaySetVolume(x) { return this.#exec('reader.view.mediaOverlay.setVolume', x) }
@@ -460,7 +473,6 @@ export const BookViewer = GObject.registerClass({
     #book
     #cover
     #data
-    #ttsPaused
     constructor(params) {
         super(params)
         utils.connect(this._view, {
@@ -656,6 +668,7 @@ export const BookViewer = GObject.registerClass({
             'start': () => this._view.mediaOverlayStart(),
             'pause': () => this._view.mediaOverlayPause(),
             'resume': () => this._view.mediaOverlayResume(),
+            'stop': () => this._view.mediaOverlayStop(),
             'backward': () => this._view.mediaOverlayPrev(),
             'forward': () => this._view.mediaOverlayNext(),
             'notify::volume': box => this._view.mediaOverlaySetVolume(box.volume),
@@ -680,7 +693,7 @@ export const BookViewer = GObject.registerClass({
             '<ctrl>f|slash': 'viewer.toggle-search',
             '<ctrl>l': 'viewer.show-location',
             '<ctrl>i|<alt>Return': 'viewer.show-info',
-            '<ctrl><alt>t': 'viewer.toggle-toc',
+            '<ctrl>t': 'viewer.toggle-toc',
             '<ctrl><alt>a': 'viewer.toggle-annotations',
             '<ctrl><alt>d': 'viewer.toggle-bookmarks',
             '<ctrl>d': 'viewer.bookmark',
@@ -695,8 +708,8 @@ export const BookViewer = GObject.registerClass({
             'plus|equal|KP_Add|KP_Equal|<ctrl>plus|<ctrl>equal|<ctrl>KP_Add|<ctrl>KP_Equal': 'view.zoom-in',
             'minus|KP_Subtract|<ctrl>minus|<ctrl>KP_Subtract': 'view.zoom-out',
             '0|1|KP_0|<ctrl>0|<ctrl>KP_0': 'view.zoom-restore',
-            'p|Page_Up': 'view.prev',
-            'n|Page_Down': 'view.next',
+            'p|Page_Up|<shift>space': 'view.prev',
+            'n|Page_Down|space': 'view.next',
             'k|Up': 'view.scroll-up',
             'j|Down': 'view.scroll-down',
             'h|Left': 'view.go-left',
@@ -726,10 +739,10 @@ export const BookViewer = GObject.registerClass({
         this._top_overlay_box.hide()
         this.#book = book
         book.metadata ??= {}
-        this.root.title = book.metadata.title ?? ''
-        this._book_title.label = book.metadata.title ?? ''
+        this._book_title.label = formatLanguageMap(book.metadata.title)
         this._book_author.label = formatAuthors(book.metadata)
-        this._book_author.visible = !!book.metadata?.author?.length
+        this._book_author.visible = !!this._book_author.label
+        this.root.title = this._book_title.label
 
         const { language: { direction } } = reader.view
         utils.setDirection(this._book_info, direction)
@@ -748,17 +761,15 @@ export const BookViewer = GObject.registerClass({
         this._navbar.loadSectionFractions(reader.sectionFractions)
         this._navbar.loadPageList(book.pageList, reader.pageTotal)
         this._navbar.loadLandmarks(book.landmarks)
-        this._navbar.setTTSType(book.media?.duration?.[''] ? 'media-overlay' : 'tts')
+        this._navbar.setTTSType(book.media?.duration ? 'media-overlay' : 'tts')
 
         const cover = await this._view.getCover()
         this.#cover = cover
         if (cover) {
-            this._book_cover.set_pixbuf(cover)
-            this._book_cover.parent.show()
-            this._book_info.height_request = 72
+            this._book_cover.set_from_pixbuf(cover)
+            this._book_cover.show()
         } else {
-            this._book_cover.parent.hide()
-            this._book_info.height_request = -1
+            this._book_cover.hide()
         }
 
         book.metadata.identifier ||= makeIdentifier(this.#file)
@@ -805,7 +816,7 @@ export const BookViewer = GObject.registerClass({
         }), { 'button-clicked': () =>
             this.#data.addAnnotation(annotation) }))
     }
-    #showSelection({ type, value, text, lang, pos: { point, dir } }) {
+    #showSelection({ type, value, text, content, lang, pos: { point, dir } }) {
         if (type === 'annotation') return new Promise(resolve => {
             this._annotation_view.scrollToCFI(value)
             const annotation = this.#data.annotations.get(value)
@@ -827,14 +838,19 @@ export const BookViewer = GObject.registerClass({
                 'highlight': () => {
                     resolved = true
                     const annotation = this.#data.annotations.get(value)
+                    // NOTE: `content` is the `Range.toString()`
+                    // whereas `text` is the `Selection.toString()`;
+                    // not sure which would be better for this use case,
+                    // but my understanding is that a `TextQuoteSelector` is
+                    // expected to be the text itself, not the rendered result
                     this.#data.addAnnotation(annotation ?? {
-                        value, text,
+                        value, text: content,
                         color: this.highlight_color,
                         created: new Date().toISOString(),
                     }).then(() => resolve('highlight'))
                 },
                 'search': () => {
-                    this._search_entry.text = text
+                    this._search_entry.text = content
                     this._search_bar.search_mode_enabled = true
                     this._flap.show_sidebar = true
                     this._search_view.doSearch()
@@ -974,10 +990,8 @@ export const BookViewer = GObject.registerClass({
         const win = new ViewPreferencesWindow({
             view_settings: this._view.viewSettings,
             font_settings: this._view.fontSettings,
-            modal: true,
-            transient_for: this.root,
         })
-        win.present()
+        win.present(this.root)
     }
     bookmark() {
         this._bookmark_view.toggle()
