@@ -432,16 +432,14 @@ GObject.registerClass({
         })
     }
     removeBook(file) {
-        const dialog = new Adw.MessageDialog({
-            transient_for: this.get_root(),
-            modal: true,
+        const dialog = new Adw.AlertDialog({
             heading: _('Remove Book?'),
             body: _('Reading progress, annotations, and bookmarks will be permanently lost'),
         })
         dialog.add_response('cancel', _('_Cancel'))
         dialog.add_response('remove', _('_Remove'))
         dialog.set_response_appearance('remove', Adw.ResponseAppearance.DESTRUCTIVE)
-        dialog.present()
+        dialog.present(this.get_root())
         dialog.connect('response', (_, response) => {
             if (response === 'remove') getBooks().delete(file)
         })
