@@ -84,6 +84,13 @@ const getCSS = ({
         }
         a:any-link {
             color: ${theme.light.link};
+            text-decoration-color: light-dark(
+                color-mix(in srgb, currentColor 20%, transparent),
+                color-mix(in srgb, currentColor 40%, transparent));
+            text-underline-offset: .1em;
+            &:hover {
+                text-decoration-color: unset;
+            }
         }
         @media (prefers-color-scheme: dark) {
             html {
@@ -112,6 +119,9 @@ const getCSS = ({
         text-align: unset;
         hyphens: unset;
     }
+    h1, h2, h3, h4, h5, h6, hgroup, th {
+        text-wrap: balance;
+    }
     pre {
         white-space: pre-wrap !important;
         tab-size: 2;
@@ -238,6 +248,13 @@ class Reader {
             ?.findLast(x => !isNaN(parseInt(x.label)))?.label
         this.style.mediaActiveClass = book.media?.activeClass
 
+        this.book.transformTarget?.addEventListener('data', ({ detail }) => {
+            detail.data = Promise.resolve(detail.data).catch(e => {
+                console.error(new Error(`Failed to load ${detail.name}`, { cause: e }))
+                return ''
+            })
+        })
+
         this.#footnoteHandler.addEventListener('before-render', e => {
             const { view } = e.detail
             view.addEventListener('link', e => {
